@@ -1230,6 +1230,13 @@ ebb_carddav_remove_contact_sync (EBookMetaBackend *meta_backend,
 	g_object_unref (contact);
 	g_free (etag);
 
+	/* Ignore not found errors, this was a delete and the resource is gone.
+	   It can be that it had been deleted on the server by other application. */
+	if (g_error_matches (local_error, SOUP_HTTP_ERROR, SOUP_STATUS_NOT_FOUND)) {
+		g_clear_error (&local_error);
+		success = TRUE;
+	}
+
 	if (local_error) {
 		ebb_carddav_check_credentials_error (bbdav, webdav, local_error);
 		g_propagate_error (error, local_error);
